package org.openmuc.core.datamanager;

/*
 * Copyright Fraunhofer ISE, 2010
 * Author(s): Michael Zillgith
 *    
 * This file is part of openMUC.
 * For more information visit http://www.openmuc.org
 * 
 * openMUC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * openMUC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with openMUC.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

import java.util.List;

import org.openmuc.core.datamanager.config.ChannelConfiguration;
import org.openmuc.core.measurement.Measurement;
import org.openmuc.core.measurement.ChannelIdentifier;
import org.openmuc.core.measurement.State;
import org.openmuc.core.measurement.Value;

/**
 * 
 * Provides an interface to access measurement and control data
 * 
 * @author mzillgit
 *
 */
public interface DataStorage {
	public ChannelIdentifier getDataId();
	public String getLabel();
	public Value getCurrentValue();
	
	/**
	 * Get a Value object for the specified timestamp.
	 * 
	 * @param timestamp
	 * @return a Value object or null if none was found.
	 */
	public Value getValue(long timestamp);
	
	public void setValue(double value);
	
	/**
	 * Get a list of Value objects (time serie).
	 * 
	 * @param startTime
	 * @return a list of Value objects or an empty list if no Value has been found.
	 */
	public List<Value> getValues(long startTime);
	
	/**
	 * Get a list of Value objects (time serie).
	 * 
	 * @param startTime
	 * @param endTime
	 * @return a list of Value objects or an empty list if no Value has been found.
	 */
	public List<Value> getValues(long startTime, long endTime);
	
	public long getLastUpdateTime();
	
	public State getState();
	
	public void setState(State state); 
	
	public Double getUpperLimit();
	
	public Double getLowerLimit();
	
	public ChannelConfiguration getConfiguration();
	
	public void addListener(DataStorageListener listener);
	public boolean addMeasurement(Measurement msr);
}
