package org.openmuc.core.datamanager;

/*
 * Copyright Fraunhofer ISE, 2010
 * Author(s): Michael Zillgith
 *    
 * This file is part of openMUC.
 * For more information visit http://www.openmuc.org
 * 
 * openMUC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * openMUC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with openMUC.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

import java.io.IOException;
import java.util.List;

import org.openmuc.core.datamanager.config.ChannelConfiguration;
import org.openmuc.core.measurement.Value;

public interface DatabaseInterface {

	/**
	 * Add a new item (measurement channel) to the database
	 * 
	 * @param channel unique identifier of the item
	 * @throws IOException
	 */
	public void addChannel(ChannelConfiguration channel) throws IOException;

	/**
	 * Insert a value into the database.
	 * 
	 * @param label label unique identifier of the item
	 * @param reading a Value object
	 * @throws IOException Will be thrown if there is a problem with
	 *         the underlying database layer
	 */
	public void insertValue(String label, Value reading)
			throws IOException;

	/**
	 * Gets all values (time series) of a channel from startTime until now.
	 * It is the same as the call
	 * getValues(label, startTime, System.currentTimeMillies)
	 * 
	 * @param label unique identifier of the item
	 * @param startTime Time of the first value in the time series in ms since epoche.
	 * @return A List of value objects or an empty list if now matching
	 * 			object have been found.
	 */
	public List<Value> getValues(String label, long startTime) throws IOException;

	/**
	 * Gets all values (time series) of a channel from startTime until endTime.
	 *
	 * @param label unique identifier of the item
	 * @param startTime Time of the first value in the time series in ms since epoche.
	 * @param endTime Time of the last value in the time series in ms since epoche
	 * @return A List of value objects or an empty list if now matching
	 * 			object have been found.
	 */
	public List<Value> getValues(String label, long startTime, long endTime) throws IOException;

	/**
	 * Gets a single value with a specified timestamp from the database.
	 * 
	 * @param label unique identifier of the item
	 * @param timestamp timestamp of the value (ms since epoche)
	 * @return a Value object if to has been found or null
	 */
	public Value getValue(String label, long timestamp) throws IOException;
}