package org.openmuc.core.datamanager.config;

import java.util.List;
import java.util.Map;

import org.openmuc.core.driver.DeviceURI;
import org.openmuc.core.measurement.ChannelIdentifier;

public interface DeviceConfiguration {

	public abstract ChannelConfiguration createChannel(String label);

	public abstract void deleteChannel(String label);

	public abstract String getMeterID();

	public abstract String getVendorID();

	public abstract String getDeviceID();

	public abstract String getVersion();

	public abstract DeviceURI getUrl();

	/**
	 * Returns an immutable list of registers
	 */
	public abstract List<ChannelConfiguration> getChannels();

	public abstract ChannelConfiguration getChannelByLabel(String label);

	public abstract List<ChannelIdentifier> getChannelIdentifierList();

	public abstract String getDescription();

	/***
	 * Return the sampling period for this device
	 * 
	 * @return sampling period in ms.
	 */
	public abstract long getSamplingPeriod();

	public abstract void setDescription(String description);

	public abstract void setVendorID(String vendorID);

	public abstract void setDeviceID(String deviceID);

	public abstract void setVersion(String version);

	public abstract String getName();

	public abstract void setName(String name);

	public abstract void setMeterUrl(String meterUrl);

	/**
	 * Set the sampling period for this device
	 * 
	 * @param period
	 *            sampling period in ms.
	 */
	public abstract void setSamplingPeriod(long period);

	public abstract String getProperty(String name);

	public abstract Map<String, String> getPropertyList();

	public abstract void setProperty(String name, String value);

	public abstract void deleteProperty(String name);

}