package org.openmuc.core.driver;

/*
 * Copyright Fraunhofer ISE, 2010
 * Author(s): Michael Zillgith
 *    
 * This file is part of openMUC.
 * For more information visit http://www.openmuc.org
 * 
 * openMUC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * openMUC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with openMUC.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

import java.io.IOException;
import java.util.List;

import org.openmuc.core.measurement.ChannelIdentifier;
import org.openmuc.core.measurement.Measurement;

public interface DeviceDriverManager {

	/**
	 * 
	 * Delegates a readDeviceAsync request to the appropriate communication driver
	 * 
	 * @param url
	 * @param list
	 * @param callback
	 * @param handle
	 * @throws NoSuchDriverException
	 * @throws IOException
	 */
	public abstract void readDeviceAsync(DeviceURI url, List<ChannelIdentifier> list, DataReceivedListener callback,
			Object handle) throws NoSuchDriverException, IOException; 

	/**
	 * 
	 * Delegates a writeChannel request for the given channel and value to the appropriate
	 * communication driver.
	 * 
	 * @param url
	 * @param channel
	 * @param value
	 * @throws NoSuchDriverException
	 * @throws MethodNotImplementedException
	 */
	public abstract void writeChannel(DeviceURI url, ChannelIdentifier channel, double value)
			throws NoSuchDriverException, MethodNotImplementedException, IOException;

	/**
	 * Returns a list of connected Devices to this busURI
	 * 
	 * @param uri
	 * @throws MethodNotImplementedException
	 * @throws NoSuchDriverException
	 * @throws IOException
	 */
	public abstract List<DeviceURI> getConnectedDevices(BusURI uri) throws MethodNotImplementedException,
			NoSuchDriverException, IOException;

	/**
	 * Returns a List of Channels for a given Device.
	 * 
	 * @param uri
	 * @throws MethodNotImplementedException
	 * @throws NoSuchDriverException
	 * @throws IOException
	 */
	public abstract List<ChannelInformation> getDeviceChannels(DeviceURI uri) throws MethodNotImplementedException,
			NoSuchDriverException, IOException;

	/**
	 * Immediately reads a Device and returns measured Results
	 * 
	 * @param uri
	 * @param channels
	 * @throws MethodNotImplementedException
	 * @throws IOException
	 */
	public List<Measurement> readDeviceSync(DeviceURI uri, List<ChannelIdentifier> channels)
			throws MethodNotImplementedException, IOException;

}