package org.openmuc.core.measurement;

/*
 * Copyright Fraunhofer ISE, 2010
 * Author(s): Michael Zillgith
 *    
 * This file is part of openMUC.
 * For more information visit http://www.openmuc.org
 * 
 * openMUC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * openMUC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with openMUC.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

/**
 * Class representing a single metered value.<br>
 * Value contains <b>value</b> : <b>timestamp</b> : <b>flag</b>
 */
public class Value {
	private double value;
	private long timestamp;
	private State state;

	public void setValue(double value) {
		this.value = value;
	}

	/**
	 * Creates a new Value Object with:
	 * 
	 * @param value
	 *            Measurement Value
	 * @param timestamp
	 *            Measurement Timestamp
	 */
	public Value(double value, long timestamp) {
		/*
		 * No Flag -> Default 0x00
		 */
		this(value, timestamp, State.valid);
	}

	/**
	 * Creates a new Value Object with:
	 * 
	 * @param value
	 *            Measurement Value
	 * @param timestamp
	 *            Measurement Timestamp
	 * @param state
	 *            Measurement Flag (Default: 0x00 if not specified)
	 */
	public Value(double value, long timestamp, State state) {
		super();
		this.value = value;
		this.timestamp = timestamp;
		this.state = state;
	}

	/**
	 * @return metered value
	 */
	public double getValue() {
		return value;
	}

	/**
	 * @return timestamp from metered value
	 */
	public long getTimestamp() {
		return timestamp;
	}

	/**
	 * @return flag from metered value
	 */
	public State getState() {
		return this.state;
	}

	/**
	 * Sets the point of measurement.
	 * 
	 * @param timestamp
	 */
	public void setTimestamp(long timestamp) {
		this.timestamp = timestamp;
	}
}
