package org.openmuc.core.obis;

/*
 * Copyright Fraunhofer ISE, 2010
 * Author(s): Michael Zillgith
 *    
 * This file is part of openMUC.
 * For more information visit http://www.openmuc.org
 * 
 * openMUC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * openMUC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with openMUC.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

import java.util.Vector;

/**
 * 
 * Representation of OBIS formated data according to IEC 62065-21/-61
 * 
 * Usage: Parse message elements of (electricity) meters that speak protocols
 * according to IEC 62056-21 (formerly IEC 1107)
 * 
 * Each OBISData object consists of one OBISCode object for self-description and
 * one or more OBISValue objects.
 * 
 * @author mzillgit
 * 
 */
public class OBISData {
	private OBISCode code = null;
	private Vector<OBISValue> values = null;

	public OBISCode getOBISCode() {
		return code;
	}

	public Vector<OBISValue> getOBISValues() {
		return values;
	}

	@Deprecated
	public OBISValue getValue() {
		if (values != null) {
			return values.elementAt(0);
		}
		else
			return null;
	}

	/**
	 * 
	 * Parse OBIS formated strings
	 * 
	 * @param obisData
	 *            containing OBIS code and data
	 */
	public OBISData(String obisData) {
		int pos;
		int pos2;
		String obisCode;
		String parseString = obisData;
		String date;

		pos = Util.containsChar(parseString, '(');

		if (pos != 0) {
			obisCode = parseString.substring(0, pos - 1);
			pos2 = Util.containsChar(parseString, ')');

			date = parseString.substring(pos, pos2 - 1);
			code = new OBISCode(obisCode);
			values = new Vector<OBISValue>();
			values.add(new OBISValue(date));

			while (pos2 < parseString.length()) {
				parseString = parseString.substring(pos2 + 1);
				pos = Util.containsChar(parseString, '(');
				pos2 = Util.containsChar(parseString, ')');
				date = parseString.substring(pos, pos2 - 1);
				values.add(new OBISValue(date));
			}

			System.out.println("OBIS-Code: " + code.toString() + " data: " + date);
		}
	} /* OBISData(String obisData) */
}
