package org.openmuc.core.collector;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Set;

import org.openmuc.core.datamanager.Authentication;
import org.openmuc.core.datamanager.DataManager;
import org.openmuc.core.datamanager.DataStorage;
import org.openmuc.core.datamanager.config.ConfigurationService;
import org.openmuc.core.datamanager.config.DeviceConfiguration;
import org.openmuc.core.measurement.ChannelIdentifier;

public class Configuration {

	private HashMap<String, List<DeviceConfiguration>> driverMeterMapping;
	private HashMap<DeviceConfiguration, List<DataStorage>> meterStorageMapping;
	
	public Configuration(ConfigurationService configuration, DataManager dataManager) 
	{
		List<DeviceConfiguration> meters = configuration.getDevices();
		
		driverMeterMapping = new HashMap<String, List<DeviceConfiguration>>();
		meterStorageMapping = new HashMap<DeviceConfiguration, List<DataStorage>>();
		
		/* Order meters by driver */
		for (DeviceConfiguration meter : meters) {
			if (meter.getUrl() == null) continue; 
			
			String driver = meter.getUrl().getDriverID();
			
			List<DeviceConfiguration> metersHoldByDriver = driverMeterMapping.get(driver);
			
			if (metersHoldByDriver == null) {
				metersHoldByDriver = new ArrayList<DeviceConfiguration>();
				driverMeterMapping.put(driver, metersHoldByDriver);
			}
			
			Authentication auth = new Authentication("root", "root");
			
			metersHoldByDriver.add(meter);
			
			List<DataStorage> dataStoragesHoldByMeter = new ArrayList<DataStorage>();
			
			meterStorageMapping.put(meter, dataStoragesHoldByMeter);
			
			for (ChannelIdentifier mdi : meter.getChannelIdentifierList()) {
				DataStorage dataStorage = dataManager.getDataStorage(mdi.getLabel(), auth);
				dataStoragesHoldByMeter.add(dataStorage);
			}
			
		}
		
	}
	
	public List<DeviceConfiguration> getMeterListForDriver(String driverId) {
		return driverMeterMapping.get(driverId);
	}
	
	public Set<String> getDrivers() {
		return driverMeterMapping.keySet();
	}
	
	public List<DataStorage> getStoragesForMeter(DeviceConfiguration meter) {
		return meterStorageMapping.get(meter);
	}
	
}
