package org.openmuc.core.datamanager.impl;

/*
 * Copyright Fraunhofer ISE, 2010
 * Author(s): Michael Zillgith
 *    
 * This file is part of openMUC.
 * For more information visit http://www.openmuc.org
 * 
 * openMUC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * openMUC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with openMUC.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

import java.util.Dictionary;
import java.util.Hashtable;

import org.apache.log4j.Logger;

import org.openmuc.core.datamanager.DataManager;
import org.openmuc.core.datamanager.config.ConfigurationService;
import org.openmuc.core.driver.DeviceDriverManager;
import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;

public class Activator implements BundleActivator {

	private static Logger logger = Logger.getLogger(Activator.class);
	
	private DataManagerImpl dataManager;
	private Configuration configuration;
			
	public void start(BundleContext context) throws Exception {
		DeviceDriverManager driverManager;
		
		DatabaseLayerFactory.setContext(context);
		
		/* get the meter manager service */
		ServiceReference serviceRef = 
			context.getServiceReference(DeviceDriverManager.class.getName());
		
		driverManager = (DeviceDriverManager) context.getService(serviceRef);
		
		if (driverManager == null) {
			logger.error("No meter driver manager service present");
			throw new Exception("No meter driver manager found");
		}
		
		String configFileName = System.getProperty("org.openmuc.core.datamanager.configfile");
		
		/* For compatibility to older versions */
		if (configFileName == null)
			configFileName = System.getProperty("org.openmuc.metering.configfile");
			
		
		configuration = new Configuration(configFileName);
		
		/* Create DataManagerImpl */
		dataManager = new DataManagerImpl(context, driverManager, configuration);
		
		configuration.setDataManager(dataManager);

		/* Register DataManager service */
		Dictionary<String, String> props = new Hashtable<String,String>();
		context.registerService(DataManager.class.getName(), dataManager, props);
		
		/* Register Configuration service */
		props = new Hashtable<String,String>();
		context.registerService(ConfigurationService.class.getName(),
				configuration, props);
	}

	public void stop(BundleContext context) throws Exception {
	}
	
}
