package org.openmuc.mux.dbprovider;

/*
 * Copyright Fraunhofer ISE, 2010
 * Author(s): Simon Fey
 *    
 * This file is part of openMUC.
 * For more information visit http://www.openmuc.org
 * 
 * openMUC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * openMUC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with openMUC.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

import java.io.IOException;
import java.util.HashMap;
import java.util.List;

import org.apache.log4j.Logger;
import org.openmuc.core.datamanager.DatabaseInterface;
import org.openmuc.core.datamanager.config.ChannelConfiguration;
import org.openmuc.core.measurement.Value;
import org.openmuc.mux.dbprovider.slotsdb.SlotsDatabase;
import org.openmuc.mux.dbprovider.slotsdb.FileObjectProxy;


public class SlotsDatabaseLayer implements DatabaseInterface {
	
	private static Logger logger = Logger.getLogger(SlotsDatabaseLayer.class);
	
	private FileObjectProxy fop;
	private HashMap<String,Integer> hm;

	public SlotsDatabaseLayer() {
		String rootFolder = SlotsDatabase.DB_ROOT_FOLDER;
		if (rootFolder == null) {
			rootFolder = SlotsDatabase.DEFAULT_DB_ROOT_FOLDER;
			logger.info(System.getProperty(SlotsDatabase.class.getPackage().getName().toLowerCase() + ".dbfolder not set. Storing to " + SlotsDatabase.DEFAULT_DB_ROOT_FOLDER));
		}
		
		fop = new FileObjectProxy(rootFolder);
		hm = new HashMap<String,Integer>();
	}

	public void addChannel(ChannelConfiguration channel) throws IOException {
		hm.put(channel.getLabel(), channel.getStoringPeriod()); //storing period in Seconds(!)
	}

	public Value getValue(String label, long timestamp) throws IOException {
		return fop.read(label, timestamp);
	}

	public List<Value> getValues(String label, long startTime) throws IOException {
		return fop.read(label, startTime, System.currentTimeMillis());
	}

	public List<Value> getValues(String label, long startTime, long endTime) throws IOException {
		return fop.read(label, startTime, endTime);
	}

	public void insertValue(String label, Value reading) throws IOException {
		fop.appendValue(label, reading.getValue(), reading.getTimestamp(), (byte)reading.getState().getCode(), hm.get(label)*1000);
	}
}
