package org.openmuc.mux.dbprovider.slotsdb;

/*
 * Copyright Fraunhofer ISE, 2010
 * Author(s): Simon Fey
 *    
 * This file is part of openMUC.
 * For more information visit http://www.openmuc.org
 * 
 * openMUC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * openMUC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with openMUC.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Calendar;

import javax.swing.*;

import org.openmuc.core.measurement.Value;

/**
 * Class providing a graphical UI to view the content of a .opm file
 * @author sfey
 *
 */
public class SlotsDBVisualizer extends JFrame{
	private static final long serialVersionUID = 1L;
	JFileChooser fc = new JFileChooser();
	File file;
	String[][] rowData = {{"0", "0", "0"}};
	String[] columnNames = { "Time", "Value", "State" };
	JTable table = new JTable( rowData, columnNames );
	JScrollPane content = new JScrollPane(table);
	
	public SlotsDBVisualizer() {
		
		JTextField   fileNameField  = new JTextField(15);
		fileNameField.setEditable(false);
		
        JMenuBar menuBar  = new JMenuBar();
        JMenu    fileMenu = new JMenu("File");
        JMenuItem openItem = new JMenuItem("Open");
        openItem.addActionListener(new openFileListener());
	
		menuBar.add(fileMenu);
		fileMenu.add(openItem);

		this.setJMenuBar(menuBar);
		this.setContentPane(content);
		this.setTitle(SlotsDatabase.FILE_EXTENSION + " File Viewer");
		this.setDefaultCloseOperation( JFrame.EXIT_ON_CLOSE );
		this.pack();
		this.setLocationRelativeTo(null);
		
	}

	class openFileListener implements ActionListener{

		public void actionPerformed(ActionEvent e) {
			int ret = fc.showOpenDialog(SlotsDBVisualizer.this);
			if (ret==JFileChooser.APPROVE_OPTION) {
				file = fc.getSelectedFile();
				java.util.List<Value> res = null;
				try {
					FileObject fo = new FileObject(file);
					res = fo.readFully();
				} catch (IOException e1) {
					e1.printStackTrace();
				}
				if(res!=null) {
					String[][] tblData = new String[res.size()][3];
					Calendar cal = Calendar.getInstance();
					for(int i = 0; i<res.size(); i++) {
						cal.setTimeInMillis(res.get(i).getTimestamp());
						
						tblData[i][0] = (new SimpleDateFormat("yyyy-MM-dd HH:mm:ss")).format(cal.getTime());
						tblData[i][1] = Double.toString(res.get(i).getValue());
						tblData[i][2] = Integer.toString(res.get(i).getState().getCode());
					}
					table = new JTable( tblData, columnNames );
					content = new JScrollPane(table);
					SlotsDBVisualizer.this.setContentPane(content);
					SlotsDBVisualizer.this.invalidate();
					SlotsDBVisualizer.this.validate();
				}
			}
		}
	}
	
	public static void main( String[] args )  {
        JFrame window = new SlotsDBVisualizer();
        window.setVisible(true);
	}
}
