package org.openmuc.mux.dbprovicer.slotsdb.test;

/*
 * Copyright Fraunhofer ISE, 2010
 * Author(s): Simon Fey
 *    
 * This file is part of openMUC.
 * For more information visit http://www.openmuc.org
 * 
 * openMUC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * openMUC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with openMUC.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

import java.io.DataOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;

import org.openmuc.mux.dbprovider.slotsdb.SlotsDatabase;
import org.openmuc.mux.dbprovider.slotsdb.FileObjectList;

import junit.framework.TestCase;

public class FileObjectListTest extends TestCase {
	public void setUp() throws Exception{
		super.setUp();
		File root = new File(SlotsDatabase.DB_TEST_ROOT_FOLDER);

		if(!root.exists())
			root.mkdirs();
		
		/*
		 * Deleting all Slots Database Files in JUnit Test folder
		 */
		for (File toDelete : root.listFiles()) {
			if (toDelete.isFile() && toDelete.getName().contains(SlotsDatabase.FILE_EXTENSION))
				toDelete.delete();
		}
	}
	
	public void testCreateFolderList() {
		/*
		 * setUp Folder
		 */
		for(int i=0; i<5;i++) {
			File file = new File(SlotsDatabase.DB_TEST_ROOT_FOLDER + "test" + i + SlotsDatabase.FILE_EXTENSION);
			try {
				if (!file.getParentFile().exists())
					file.getParentFile().mkdirs();
				file.createNewFile();
				DataOutputStream dos = new DataOutputStream(new FileOutputStream(file));
				dos.writeLong(100000 + i*1000);
				dos.writeLong(5000);
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		
		FileObjectList fol = null;
		try {
			fol = new FileObjectList(SlotsDatabase.DB_TEST_ROOT_FOLDER);
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		//Tests if Folder List is sorted
		for(int i = 0; i<fol.size();i++)
			assertEquals(100000+i*1000, fol.get(i).getStartTimeStamp());
		
		//Tests if current is latest
		assertEquals(104000, fol.getCurrentFileObject().getStartTimeStamp());
	}
	
	public void tearDown() throws Exception{
		super.tearDown();
		setUp();
	}

}
