package org.openmuc.ehz.iec62056_21;

/*
 * Copyright Fraunhofer ISE, 2011
 * Author(s): Frederic Robra
 *    
 * This file is part of openMUC.
 * For more information visit http://www.openmuc.org
 * 
 * openMUC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * openMUC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with openMUC.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

import gnu.io.CommPortIdentifier;
import gnu.io.PortInUseException;
import gnu.io.SerialPort;
import gnu.io.UnsupportedCommOperationException;

import java.io.DataInputStream;
import java.io.IOException;
import java.util.Enumeration;

import org.openmuc.ehz.exception.NotFoundException;


public class IECReceiver {
//	public final static int PROTOCOL_NORMAL = 0;
//	public final static int PROTOCOL_SECONDARY = 1;
//	public final static int PROTOCOL_HDLC = 2;
//	
//	public final static int MODE_DATA_READOUT = 0;
//	public final static int MODE_PROGRAMMING = 1;
//	public final static int MODE_BINARY_HDLC = 2;,
	
	private CommPortIdentifier portId;
	private SerialPort serialPort;
	private byte[] msgBuffer = new byte[10000];
	private byte[] inputBuffer = new byte[2000];
	private DataInputStream inStream;
	
	private class Timeout extends Thread {
		private long time;
		private boolean end;
		public Timeout(long msTimeout) {
			this.time = msTimeout;
			this.end = false;
		}
		
		@Override
		public void run() {
			try {
				Thread.sleep(time);
			}
			catch (InterruptedException e) { }
			end = true;
			return;
		}
		
		public boolean isEnd() {
			return end;
		}
	}
		
	public IECReceiver(String iface) throws NotFoundException {
		Enumeration<?> portList;
		portList = CommPortIdentifier.getPortIdentifiers();
		while (portList.hasMoreElements()) {
			portId = (CommPortIdentifier) portList.nextElement();
			
			if (portId.getPortType() == CommPortIdentifier.PORT_SERIAL) {
				if (portId.getName().equals(iface)) {
					try {
						serialPort = (SerialPort) portId.open("ehz_connector", 2000);

						serialPort.setSerialPortParams(9600, 
								SerialPort.DATABITS_7, 
								SerialPort.STOPBITS_1, 
								SerialPort.PARITY_EVEN);
						
						serialPort.setOutputBufferSize(1);
						
						inStream = new DataInputStream(serialPort.getInputStream());
						
						if (inStream.available() > 0) {		
							inStream.read(inputBuffer);
						}
						try {Thread.sleep(100);} catch (InterruptedException e) { }
					}
					catch (PortInUseException e) {
						throw new NotFoundException("Port " + iface + " in use!");
					}
					catch (UnsupportedCommOperationException e) {
						throw new NotFoundException("Error setting communication parameters!");
					}
					catch(IOException e) {
						throw new NotFoundException("Cannot catch output stream!");
					}
					
					break;
				}
			}
		}
	}
	
	public byte[] receiveMessage(long msTimeout) throws IOException {
		Timeout time = new Timeout(msTimeout);
		time.start();
		
		int bufferIndex = 0;
		boolean start = false;
		boolean end = false;
		inStream.skip(inStream.available());					// inStream to current state

		do {
			if (inStream.available() > 0) {
				int read = inStream.read(inputBuffer);
				
				for (int i = 0; i < read; i++) {
					byte input = inputBuffer[i];
					if (input == '/' && !start) {
						start = true;
						bufferIndex = 0;
					}
					msgBuffer[bufferIndex] = input;
					bufferIndex++;
					if (input == '!' && start) {
						end = true;
					}
				}
			}
			if (end && start) {
				break;
			}
			
			try {
				Thread.sleep(50);
			}
			catch (InterruptedException e) { }
		} while (!time.isEnd());


		if (time.isEnd()) {
			throw new IOException("Timeout");
		}
		
		byte[] frame = new byte[bufferIndex];
		
		for (int i = 0; i < bufferIndex; i++) {
			frame[i] = msgBuffer[i];
		}

		return frame;
	}
	

	
	public void changeBaudrate(int baudrate) {
		try {
			System.out.println("Change baudrate to: "  + baudrate);
			serialPort.setSerialPortParams(baudrate, SerialPort.DATABITS_7, 
				SerialPort.STOPBITS_1, SerialPort.PARITY_EVEN);
		}
		catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	public void close() {
		serialPort.close();
	}
	
	@Override
	protected void finalize() throws Throwable {
		this.close();
		super.finalize();
	}
	
}
