package org.openmuc.modbus;

/*
 * Copyright Fraunhofer ISE, 2010
 * Author(s): Simon Fey
 *    
 * This file is part of openMUC.
 * For more information visit http://www.openmuc.org
 * 
 * openMUC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * openMUC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with openMUC.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

import java.io.IOException;
import java.util.List;
import java.util.Vector;
import java.util.concurrent.Executor;
import java.util.concurrent.Executors;
import java.util.concurrent.ThreadFactory;

import org.openmuc.core.driver.BusURI;
import org.openmuc.core.driver.ChannelInformation;
import org.openmuc.core.driver.DataReceivedListener;
import org.openmuc.core.driver.DeviceDriverInterface;
import org.openmuc.core.driver.DeviceURI;
import org.openmuc.core.driver.MethodNotImplementedException;
import org.openmuc.core.driver.NoSuchInterfaceException;
import org.openmuc.core.measurement.ChannelIdentifier;
import org.openmuc.core.measurement.Measurement;

public class ModbusDriverImpl implements DeviceDriverInterface {
	private static int instanceCount = 0;
	private int instanceID = 0;
	private int threadCount = 0;
	public boolean debug = false;
	
	private Vector<ModbusDevice> devicePool = new Vector<ModbusDevice>();
	private Executor executor;
	
	public ModbusDriverImpl() {
		instanceID = instanceCount++;
		
		/* Create new thread pool */
		executor = Executors.newCachedThreadPool(new ThreadFactory() {
			public Thread newThread(Runnable arg0) {
				if (Boolean.valueOf(System.getProperty("org.openmuc.modbus.debug")).booleanValue()) {
					System.out.println("MODBUS DEBUG: Creating new modbus Thread: " + instanceID + " " + threadCount+1);
				}
				return new Thread(arg0, "modbusthreadpool-" + instanceID + "-" + threadCount++);
			}
		});
	}

	public String getDriverID() {
		return "modbus";
	}

	public void writeChannel(DeviceURI url, ChannelIdentifier register,
			double value) throws MethodNotImplementedException, IOException {
		ModbusDevice dev = null;
		
		/* Check if device is already in device pool */	
		for (ModbusDevice device : devicePool) {
			if (device.getUrl().getAddress().equals(url.getAddress())) {
				dev = device;
				break;
			}
		}
		
		/* Add new device to device pool */
		if (dev == null) {
			try {
				dev = new ModbusDevice(url);
				devicePool.add(dev);
			} catch (Exception e) {
				System.err.println("Cannot connect to " + url.toString());
				return;
			}
		}
		
		dev.writeOutputRegister(register, value);
	}

	
	
	
	public List<DeviceURI> getDeviceList(BusURI url)
			throws MethodNotImplementedException {
		return null;
	}

	public void readDeviceAsync(DeviceURI url,
			List<ChannelIdentifier> registers,
			DataReceivedListener callback, Object handle)
			throws MethodNotImplementedException 
	{
		
		ModbusDevice dev = null;
		
		/* Check if device is already in device pool */	
		for (ModbusDevice device : devicePool) {
			if (device.getUrl().getAddress().equals(url.getAddress())) {
				dev = device;
				break;
			}
		}
		
		/* Add new device to device pool */
		if (dev == null) {
			try {
				dev = new ModbusDevice(url);
				devicePool.add(dev);
			} catch (Exception e) {
				System.err.println("Cannot connect to " + url.toString());
				return;
			}
		}
		
		executor.execute(new ReadoutTask(dev, registers, callback, handle));
	}

	public void configureInterface(String iface, String configStr)
			throws NoSuchInterfaceException, MethodNotImplementedException,
			IOException {
		// TODO Auto-generated method stub
		
	}

	public List<String> getSupportedOperations() {
		// TODO Auto-generated method stub
		return null;
	}

	public List<Measurement> readDeviceSync(DeviceURI url,
			List<ChannelIdentifier> channels)
			throws MethodNotImplementedException {
		// TODO Auto-generated method stub
		return null;
	}

	public List<ChannelInformation> getDeviceChannels(DeviceURI url)
			throws MethodNotImplementedException, IOException {
		throw new MethodNotImplementedException();
	}
}


