package org.openmuc.modbus;

/*
 * Copyright Fraunhofer ISE, 2010
 * Author(s): Simon Fey
 *    
 * This file is part of openMUC.
 * For more information visit http://www.openmuc.org
 * 
 * openMUC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * openMUC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with openMUC.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

public class ModbusObjectLocator {
	
	private int registerAddress;
	private int registerCount;
	private String registerDataType;
	
	private DataType dataType = DataType.dt_notype;
	public enum DataType {
		dt_float, dt_double, dt_int, dt_short, dt_long, dt_notype;
		public static DataType toDataType(String str) {
			try {
				return valueOf("dt_" + str);
			} catch (Exception e) {
				return dt_notype;
			}
		}
	}
	
	/**
	 * @locator Modbus Device Locator "registerAddress:registerDatatype" e.g.: "1317:float" 
	 */
	public ModbusObjectLocator (String locator) throws MalformedObjectLocatorException {
		if (locator.matches("[0-9]+?:[a-zA-Z0-9]+?")) { //RegEx for int:string
			int registerDataTypeOffset = locator.indexOf(':');
			this.registerAddress = Integer.parseInt(locator.substring(0,registerDataTypeOffset));
			this.registerDataType = locator.substring(registerDataTypeOffset+1);
			
			dataType = DataType.toDataType(this.registerDataType);
			if (dataType != DataType.dt_notype) {
				switch (dataType) {
				case dt_float:
					this.registerCount = 2; break;
				case dt_double:
					this.registerCount = 4; break;
				case dt_int:
					this.registerCount = 2; break;
				case dt_short:
					this.registerCount = 1; break;
				case dt_long:
					this.registerCount = 4; break;
				}
			} else {
				throw new MalformedObjectLocatorException("Unknown Data Type in ObjectLocator. Known types: float, double, int, short, long");
			}
		} else {
			throw new MalformedObjectLocatorException("ObjectLocator doesn't match int:string e.g.: 1000:float");
		}
	}
	
	/**
	 * @return Register Address, which has been parsed from the objectLocator String
	 */
	public int getRegisterAddress() {
		return registerAddress;
	}
	
	/**
	 * @return Register Count. Modbus Devices use sequenced registers to store data (2 registers for a float, 1 for a short, etc.)
	 */
	public int getRegisterCount() {
		return registerCount;
	}
	
	/**
	 * @return Register Data Type. Which can be a float, short, int32 etc.
	 */
	public DataType getRegisterDataType() {
		return dataType;
		//return registerDataType;
	}
	
}
