package org.openmuc.sunnywebbox;

/*
 * Copyright Fraunhofer ISE, 2010
 * Author(s): Simon Fey
 *    
 * This file is part of openMUC.
 * For more information visit http://www.openmuc.org
 * 
 * openMUC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * openMUC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with openMUC.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.SocketException;
import java.net.SocketTimeoutException;
import java.net.UnknownHostException;

import org.apache.log4j.Logger;

/**
 * Class implementing a UDP Connection to a SMA Sunny Webbox
 * SunnyWebboxConnection provides a sendJSONRequest function to send and receive serialized JSON Strings
 * @author sfey
 *
 */
public class SunnyWebboxConnection {
	private static DatagramSocket socket;
	private DatagramPacket packet_send;
	private DatagramPacket packet_recv;
	private InetAddress addr;
	private int port;
	private byte[] buf;
	
	private static Logger logger = Logger.getLogger(SunnyWebboxConnection.class);
	
	public SunnyWebboxConnection(String server) throws UnknownHostException, SocketException {
			logger.trace("Initializing Connection to: " + server);
			this.port = SunnyWebbox.port;
			this.addr = InetAddress.getByName(server);
			if(socket==null) {
				this.socket = new DatagramSocket(this.port);
				socket.setSoTimeout(SunnyWebbox.timeout_millis);
			}
			buf = new byte[SunnyWebbox.BUFFER_SIZE];
	}
	
	/**
	 * Send a serialized JSON request and receive its response as a serializes JSON String
	 * @param toSend
	 * @return
	 * @throws IOException
	 */
	public synchronized String sendJSONRequest(String toSend) throws IOException {
			logger.trace("Sending JSON String: " + toSend);
		buf = expand(toSend.getBytes()); 
		packet_send = new DatagramPacket(buf, buf.length, addr, port);
		packet_recv = new DatagramPacket(new byte[100000], 100000);
		try {
		socket.send(packet_send);
		socket.receive(packet_recv);
		} catch (SocketTimeoutException ste){
			System.err.println("JSON Request timed out. SunnyWebbox: " + addr.getHostAddress() + " unreachable.");
			throw ste;
		}
		String received = new String(collapse(packet_recv.getData()));
			logger.trace("Received JSON Response: " + received);
		return received.trim();
	}
	
	/**
	 * Expands the String to send to a String the Webbox understands.
	 * SunnyWebbox needs a "0" after each character.
	 * @param buf
	 * @return
	 */
	private static byte[] expand(byte[] buf) {
		byte[] buf2 = new byte[buf.length*2+1];
		for (int i = 0; i < buf.length; i++) {
			buf2[2*i]=buf[i];
			buf2[2*i+1]=0;
		}
		return buf2;
	}
	
	/**
	 * Removes the 0 between the characters.
	 * @param buf
	 * @return
	 */
	private static byte[] collapse(byte[] buf) {
		byte[] buf2 = new byte[buf.length/2];
		for(int i = 0; i< buf2.length; i++) {
			buf2[i]=buf[2*i];
		}
		return buf2;
	}
	
	public void close() {
		socket.close();
	}
}
