package org.openmuc.mux.dataserver;

/*
 * Copyright Fraunhofer ISE, 2010
 * Author(s): Michael Zillgith
 *    
 * This file is part of openMUC.
 * For more information visit http://www.openmuc.org
 * 
 * openMUC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * openMUC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with openMUC.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

import java.util.Dictionary;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Properties;

import org.apache.log4j.Logger;
import org.openmuc.core.datamanager.DataManagerListener;
import org.openmuc.core.datamanager.config.ConfigurationListener;
import org.openmuc.core.measurement.ChannelIdentifier;
import org.openmuc.core.measurement.State;
import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.service.cm.ConfigurationException;
import org.osgi.service.cm.ManagedService;

public class Activator implements BundleActivator, ManagedService, 
	DataManagerListener, ConfigurationListener 
{
	private static Logger logger = Logger.getLogger(Activator.class);

	private DataSocketServer server = null;
	private DataManagerServiceTracker dataManagerTracker;
	private Dictionary configProperties = null;
	
	/*
	 * (non-Javadoc)
	 * @see org.osgi.framework.BundleActivator#start(org.osgi.framework.BundleContext)
	 */
	public void start(BundleContext context) throws Exception {
		dataManagerTracker = new DataManagerServiceTracker(context, this);
		dataManagerTracker.open();
		
		Dictionary properties = new Hashtable();
		properties.put("service.pid", "org.openmuc.mux.dataserver");
		context.registerService(ManagedService.class.getName(), this, properties);
		
	}

	/*
	 * (non-Javadoc)
	 * @see org.osgi.framework.BundleActivator#stop(org.osgi.framework.BundleContext)
	 */
	public void stop(BundleContext context) throws Exception {
		server.stopServer();
		dataManagerTracker.close();
	}

	public void updated(Dictionary properties) throws ConfigurationException {
		this.configProperties = properties;
		
		if (properties != null) {
			restartServer(properties);
		}
		else {
			logger.info("org.openmuc.mux.dataserver: no configuration found!");
		}
	}

	private void restartServer(Dictionary properties) {
		Enumeration keys = properties.keys();
		Properties props = new Properties();

		logger.info("org.openmuc.mux.dataserver configuration elements:");
		while (keys.hasMoreElements()) {
			String key = (String) keys.nextElement();
			props.setProperty(key, (String) properties.get(key));
		}
		
		if (server != null) {
			server.stopServer();
			server = null;
			logger.info("Waiting for server to stop...");
			try {
				Thread.sleep(2000);
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		server = new DataSocketServer(dataManagerTracker, props);
		server.start();
	}

	public void updated() {
		try {updated(configProperties);} catch (ConfigurationException e) {}
	}

	public void storingDataEvent(String label, long timestamp, double value) {
	}

	public void configurationChanged() {
		restartServer(this.configProperties);
	}

	public void stateChanged(ChannelIdentifier id, State newState) {/* nop */}
	
}
