package org.openmuc.mux.dataserver;

/*
 * Copyright Fraunhofer ISE, 2010
 * Author(s): Michael Zillgith
 *    
 * This file is part of openMUC.
 * For more information visit http://www.openmuc.org
 * 
 * openMUC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * openMUC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with openMUC.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

import org.openmuc.core.datamanager.DataManager;
import org.openmuc.core.datamanager.config.ConfigurationService;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;
import org.osgi.util.tracker.ServiceTracker;

/**
 * Track the mux DataManager. The DataManager provides
 * access to the measurements data.
 * 
 * @author mzillgit
 *
 */
public class DataManagerServiceTracker extends ServiceTracker {

	private DataManager manager = null;
	private Activator activator;
	ServiceReference configServiceRef = null;

	public DataManagerServiceTracker(BundleContext bc, Activator activator) {
		super(bc, DataManager.class.getName(), null);
		this.activator = activator;
	}
	
	public DataManager getManager() {
		return manager;
	}

	@Override
	public Object addingService(ServiceReference reference) {
		manager = (DataManager) context.getService(reference);
		configServiceRef = context.getServiceReference(ConfigurationService.class.getName());
		ConfigurationService configuration = (ConfigurationService) 
			context.getService(configServiceRef);
		configuration.registerListener(activator);
		return manager;
	}

	@Override
	public void removedService(ServiceReference reference, Object service) {
		manager = null;
		context.ungetService(reference);
		context.ungetService(configServiceRef);
	}
}

