package org.openmuc.mux.writer;

import java.util.Dictionary;
import java.util.Hashtable;

import org.apache.log4j.Logger;
import org.openmuc.core.datamanager.DataManager;
import org.openmuc.core.datamanager.DataManagerListener;
import org.openmuc.core.datamanager.config.ConfigurationListener;
import org.openmuc.core.datamanager.config.ConfigurationService;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;
import org.osgi.framework.ServiceRegistration;
import org.osgi.service.cm.ConfigurationException;
import org.osgi.service.cm.ManagedService;
import org.osgi.util.tracker.ServiceTracker;

public class DataManagerTracker extends ServiceTracker 
	implements ManagedService, ConfigurationListener
{
	
	Logger logger = Logger.getLogger(DataManagerTracker.class);
	
	private DataManager dataManager = null;
	private ConfigurationService configuration =  null;
	private ServiceReference configServiceRef = null;
	private CurrentDataFileWriter fileWriter = null;
	private BundleContext context;
	
	@SuppressWarnings("rawtypes")
	private Dictionary properties = new Hashtable();
	private ServiceRegistration registration = null;
	
	public DataManagerTracker(BundleContext context) {
		super(context, DataManager.class.getName(), null);
		this.context = context;
	}

	@Override
	public Object addingService(ServiceReference reference) {
		this.dataManager = (DataManager) context.getService(reference);
		
		this.configServiceRef = context.getServiceReference(ConfigurationService.class.getName());
		this.configuration = (ConfigurationService) context.getService(configServiceRef);
		configuration.registerListener(this);
		
		startWriter();
		return this.dataManager;
	}



	@Override
	public void removedService(ServiceReference reference, Object service) {
		stopWriter();
		dataManager = null;
		context.ungetService(reference);
		context.ungetService(configServiceRef);
	}

	@Override
	public synchronized void close() {
		super.close();
		stopWriter();
	}

	public void updated(@SuppressWarnings("rawtypes") Dictionary properties) throws ConfigurationException {
		this.properties = properties;
		logger.info("updated!");
		stopWriter();
		startWriter();
	}
	
	private synchronized void startWriter() {
		logger.info("start writer");
		if (dataManager != null && fileWriter == null) {
			try {
				logger.info("new writer");
				fileWriter = new CurrentDataFileWriter(context, this, properties);
				
				Dictionary<String,String> properties = new Hashtable<String,String>();
				registration = context.registerService(DataManagerListener.class.getName(), 
						fileWriter, properties);
				fileWriter.start();
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}
	
	private synchronized void stopWriter() {
		logger.info("stop writer");
		if (fileWriter != null)  {
			if (registration != null) {
				logger.info("unregister writer");
				registration.unregister();
				registration = null;
			}
			fileWriter.setRunning(false);
			fileWriter.interrupt();
			fileWriter = null;
		}
	}

	public DataManager getDataManager() {
		return dataManager;
	}

	public void configurationChanged() {
		logger.info("restart due to configuration update");
		stopWriter();
		startWriter();
	}
	
}
