package org.openmuc.sunnywebtool.internal;

import java.io.IOException;
import java.net.SocketTimeoutException;
import java.net.URL;
import java.util.Hashtable;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.openmuc.webinterface.PluginCategory;
import org.openmuc.webinterface.PluginContext;
import org.openmuc.webinterface.ResourceLoader;
import org.openmuc.webinterface.View;
import org.openmuc.webinterface.WebInterfacePlugin;
import org.osgi.framework.BundleContext;
import org.openmuc.core.datamanager.config.ConfigurationService;
import org.openmuc.core.driver.BusURI;

public class SunnyWebToolPlugin implements WebInterfacePlugin {
	
	private BundleContext bundleContext;
	private ResourceLoader loader;
	private DeviceConfigurator deviceConfigurator;
	
	public SunnyWebToolPlugin(BundleContext context, ConfigurationService configService) {
		this.bundleContext = context;
		loader = new ResourceLoader(context);
		deviceConfigurator = new DeviceConfigurator(configService, context);
	}

	public String getApplicationAlias() {
		return "sunnywebtool";
	}

	public String getApplicationName() {
		return "Sunny Webbox Tool";
	}
	
	public String getApplicationDescription(){
		return "Tool to connect to a Sunny Webbox";
	}

	public PluginCategory getCategory() {
		return PluginCategory.TOOL;
	}

	public View getContentView(HttpServletRequest request, PluginContext context) {
		String template = "";
		
		if (context.getLocalPath().equals("/configure")) {
			String ipAddress = request.getParameter("webboxIP");
			String deviceAddress = request.getParameter("selectedDevice");
			String[] selectedChannels = request.getParameterValues("selectedChannels");
			String storageIntervall = request.getParameter("storageIntervall");

			template = loader.getResourceAsString("message.html");
			SunnyWebboxView view = new SunnyWebboxView(template);
			
			if (ipAddress != null && deviceAddress != null && selectedChannels != null) {
				deviceConfigurator.configureDevice(ipAddress, deviceAddress, selectedChannels, storageIntervall);
				view.addToContext("sunnywebboxMessage", "Configured SunnyWebbox Device: " +deviceAddress+" on IP: "+ipAddress);
			}
			else {
				if (ipAddress == null)
					view.addToContext("sunnywebboxMessage", "Error: Missing IP!");
				if (deviceAddress == null)
					view.addToContext("sunnywebboxMessage", "Error: Missing Device Address!");
				if (selectedChannels == null)
					view.addToContext("sunnywebboxMessage", "Error: No channels selected!");
				if (ipAddress == null && deviceAddress == null) {
					view.addToContext("sunnywebboxMessage", "Error: Missing IP and Device Adress");
				}
			}

			return view;
		}
		else if (context.getLocalPath().equals("/listmodules")) {
			template = loader.getResourceAsString("modulesview.html");
			SunnyWebboxView view = new SunnyWebboxView(template);
			String ip = request.getParameter("ipaddress");
			
			//DeviceInfo deviceInfo = new DeviceInfo(ip);
			
			SunnyWebbox sunnyWebbox = null;
			try {
				sunnyWebbox = new SunnyWebbox(new BusURI("sunnywebbox"+":"+ip),bundleContext);
			} catch (IOException e) {
				template = loader.getResourceAsString("message.html");
				view = new SunnyWebboxView(template);
				view.addToContext("sunnywebboxMessage", "Connection to: " + ip + " failed.");
				return view;
			}
			
			view.addToContext("SunnyWebboxIP", ip);
			view.addToContext("webboxDevices", sunnyWebbox.getDevices());

			return view;
		}
		else {
			template = loader.getResourceAsString("enterIP.html");		
			return new SunnyWebboxView(template);
		}
	}

	public Hashtable<String, String> getResources() {
		Hashtable<String,String> resources = new Hashtable<String,String>();
		
		resources.put("css", "css");
		
		return resources;
	}

	public String getMimeType(String name) {
		return null;
	}

	public URL getResource(String name) {	
		return bundleContext.getBundle().getResource(name);
	}

	public boolean handleSecurity(HttpServletRequest request,
			HttpServletResponse response) throws IOException {
		return true;
	}

}
