package org.openmuc.webconfigadmin.internal;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.util.Dictionary;
import java.util.Enumeration;
import java.util.Hashtable;

import org.osgi.framework.Bundle;
import org.osgi.framework.InvalidSyntaxException;
import org.osgi.service.cm.Configuration;
import org.osgi.service.cm.ConfigurationAdmin;

public class BundleConfiguration {
	private Configuration config;
	private boolean hasConfig;
	private ConfigurationAdmin confAdmin;
	private Bundle bundle;
	
	public BundleConfiguration(Bundle bundle, ConfigAdminTracker configAdminTracker,
								MetaTypeServiceTracker metaTypeTracker)
										throws IOException, InvalidSyntaxException {
		this.hasConfig=false;
		this.bundle=bundle;
		String location = bundle.getLocation();
		this.confAdmin = configAdminTracker.getConfAdmin();
		if (confAdmin != null) {
			Configuration[] configs = confAdmin.listConfigurations(null);
			if (configs != null) {
				for (Configuration config : configs) {
					if(config.getBundleLocation()!=null)
					{
						config = confAdmin.getConfiguration(config.getPid());
						if (config.getBundleLocation().contains(location)||
								location.contains(config.getBundleLocation())) {
							this.config=config;
							this.hasConfig=true;
						}
					}
					
				}
			}
		}
		

	}

	@SuppressWarnings("unchecked")
	public Dictionary<String, String> getBundleProperties() {
		if(hasConfig){
			return config.getProperties();
		}
		else{
			Dictionary<String, String> properties = new Hashtable<String, String>();
			return properties;
		}
	}

	public void setBundleProperties(Dictionary<String, String> bundleProperties) throws IOException {
		if(bundleProperties.size()==1){
			System.out.println("Create new config");
			config=confAdmin.createFactoryConfiguration(bundle.getSymbolicName());
			config.setBundleLocation(bundle.getLocation());
		}
		if(bundleProperties.size()>=1){
			config.update(bundleProperties);
			
			Writer output = null;
			
			String text = new String();
			Enumeration<String> keys=bundleProperties.keys();
			while(keys.hasMoreElements()){
				String key=keys.nextElement();
				if(!key.contains("service.")){
					text=text+key+"="+bundleProperties.get(key)+"\n";
				}
			}

			File file=null;
			String path=System.getProperty("bundles.configuration.location");
			if(path!=null){
				path.trim();
				if(path.endsWith("/")){
					path=path.substring(0, path.length()-1);
				}
				file = new File(path+"/services/"+config.getPid()+".properties");
			}
			else{
				file = new File("configurations/services/"+config.getPid()+".properties");
			}
			if(file.exists()){		//only write if paxconfman data available
				output = new BufferedWriter(new FileWriter(file));
				output.write(text);
				output.flush();
				output.close();
			}
		}
		else{
			try{
				config.delete();
			}
			catch(Exception e){}
		}
	}
	
	public boolean hasConfig(){
		return hasConfig;
	}
	
	public Configuration getConfig(){
		return config;
	}
	
}
