package org.openmuc.webconfigadmin.internal;

import java.util.ArrayList;
import java.util.Dictionary;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.List;

import org.openmuc.webinterface.ResourceLoader;
import org.openmuc.webinterface.View;
import org.osgi.framework.Bundle;
import org.osgi.service.metatype.AttributeDefinition;
import org.osgi.service.metatype.MetaTypeInformation;
import org.osgi.service.metatype.MetaTypeService;
import org.osgi.service.metatype.ObjectClassDefinition;

public class ConfigurationView implements View {

	private String template;
	private HashMap<String,Object> context = null;
	private Bundle bundle;
	
	
	public ConfigurationView(Bundle bundle, BundleConfiguration bundleConfiguration,
			MetaTypeServiceTracker metaTypeServiceTracker, ResourceLoader loader) 
	{
		this.template=loader.getResourceAsString("edit.html");
		context = new HashMap<String,Object>();
		this.bundle=bundle;
		
		Dictionary<String, String> bundleProperties=bundleConfiguration.getBundleProperties();
		
		//context.put("bundleProperties", bundleProperties);
		context.put("bundle", bundle);
		
		HashMap<String,HashMap<String, String>> map = new HashMap<String,HashMap<String, String>>();
		if(bundleProperties!=null){
			Enumeration<String> keys=bundleProperties.keys();
			while(keys.hasMoreElements()){
				HashMap<String, String> temp=new HashMap<String, String>();
				String key=keys.nextElement();
				temp.put("value",bundleProperties.get(key));
				temp.put("description","");
				map.put(key,temp);
				
			}
			
			/* Check for MetaTypeService */
			MetaTypeService meta = metaTypeServiceTracker.getMetaTypeService();
			
			if (meta == null) {
				System.out.println("No MetaTypeService found!");
			}
			else {
				MetaTypeInformation info = meta.getMetaTypeInformation(bundle);
				String[] pids = info.getPids();
				
				for (String pid : pids) {				
					ObjectClassDefinition ocd = info.getObjectClassDefinition(pid, null);
					
					AttributeDefinition[] ads = ocd.getAttributeDefinitions(ObjectClassDefinition.ALL);
					for (AttributeDefinition ad : ads) {
						HashMap<String, String> temp=new HashMap<String, String>();
						
						temp.put("value",bundleProperties.get(ad.getID()));
						temp.put("description",ad.getDescription());
						
						map.put(ad.getID(), temp);
					}
				}
			}
		}

		context.put("bundleProperties", map);
	}
	
	public HashMap<String, Object> getContext() {
		return context;
	}

	public List<String> getJavaScripts() {
		return new ArrayList<String>();
	}

	public List<String> getStyleSheets() {
		ArrayList<String> styleSheets = new ArrayList<String>();
		styleSheets.add("/openmuc/css/openmuc-theme/jquery-ui-1.8.14.custom.css");
		return styleSheets;
	}

	public String getTemplate() {
		return template;
	}

	public String getPage() {
		return "Configure bundle " + bundle.getSymbolicName();
	}

	public String getRedirectLocation() {
		return null;
	}

	public viewtype getViewType() {
		// TODO Auto-generated method stub
		return null;
	}
}
