package org.openmuc.webconfigadmin.internal;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.Dictionary;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.FileItemFactory;
import org.apache.commons.fileupload.FileUploadException;
import org.apache.commons.fileupload.disk.DiskFileItemFactory;
import org.apache.commons.fileupload.servlet.ServletFileUpload;
import org.openmuc.webinterface.PluginCategory;
import org.openmuc.webinterface.PluginContext;
import org.openmuc.webinterface.RedirectView;
import org.openmuc.webinterface.ResourceLoader;
import org.openmuc.webinterface.View;
import org.openmuc.webinterface.WebInterfacePlugin;
import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;
import org.osgi.framework.BundleException;

/*
 * For a president store osgi.clean must be set to false
 * 
 * 		osgi.clean=false
 * 
 * and you can not use org.ops4j.pax.configmanager
 * 
 */

public class WebConfiguratorPlugin implements WebInterfacePlugin {

	private BundleContext context;
	private ResourceLoader loader;
	private MetaTypeServiceTracker metaTypeTracker;
	private ConfigAdminTracker configAdminTracker;
	private String message=null;
	
	public WebConfiguratorPlugin(BundleContext context,
			MetaTypeServiceTracker metaTypeTracker,
			ConfigAdminTracker configAdminTracker) {
		this.context = context;
		this.metaTypeTracker = metaTypeTracker;
		this.configAdminTracker = configAdminTracker;
		loader = new ResourceLoader(context);
		message=null;
	}
	
	public Hashtable<String,String> getResources() {
		Hashtable<String,String> resources = new Hashtable<String,String>();
		
		resources.put("css", "css");
		
		return resources;
	}
	
	public String getApplicationAlias() {
		return "webconfigurator";
	}

	public String getApplicationName() {
		return "Bundle Configuration";
	}
	
	public String getApplicationDescription(){
		return "Configuration utility for the OSGI bundles.";
	}

	public View getContentView(HttpServletRequest request, PluginContext pluginContext) {
		if (pluginContext.getLocalPath().equals("/edit")) {
			Bundle bundle = getBundleById(request.getParameter("id"));
			
			BundleConfiguration bundleConfiguration=null;
			try{
				bundleConfiguration=new BundleConfiguration(bundle, configAdminTracker, metaTypeTracker);
			}
			catch(Exception e){
				message="Error: "+e.getMessage();
				return new RedirectView(pluginContext.getApplicationPath());
			}
			
			return new ConfigurationView(bundle, bundleConfiguration, metaTypeTracker, loader);			
				
		}
		else if (pluginContext.getLocalPath().equals("/changebundle")){
			String id=request.getParameter("id");
			Bundle bundle = getBundleById(id);
			try{
				BundleConfiguration bundleConfiguration=new BundleConfiguration(bundle, configAdminTracker, metaTypeTracker);
				Dictionary<String, String> bundleProperties=new Hashtable<String, String>();
				
				String[] keys = request.getParameterValues("keyList");
				if(keys!=null){
					for(String key : keys){
						bundleProperties.put(key, request.getParameter(key+"Property"));
					}
				}
				String newkey=request.getParameter("newkey");
				if(!newkey.isEmpty()){
					bundleProperties.put(newkey, request.getParameter("newkeyProperty"));
				}
				bundleConfiguration.setBundleProperties(bundleProperties);
				
				return new RedirectView(pluginContext.getApplicationPath()+"/edit?id="+id);
			}
			catch(Exception e){
				message="Error: "+e.getMessage();
				return new RedirectView(pluginContext.getApplicationPath());
			}
		}		
		else if (pluginContext.getLocalPath().equals("/install")){
			installBundle(request, pluginContext);
			return new RedirectView(pluginContext.getApplicationPath());
		}
		else if (pluginContext.getLocalPath().equals("/uninstall")) {
			Bundle bundle = getBundleById(request.getParameter("id"));
			if (bundle != null) {	
				try {
					message="Info: "+bundle.getSymbolicName()+" uninstalled";
					bundle.uninstall();
				} catch (BundleException e) {
					message="Error: "+e.getMessage();
				}	
			}
			return new RedirectView(pluginContext.getApplicationPath());		
		}
		else if (pluginContext.getLocalPath().equals("/update")) {
			Bundle bundle = getBundleById(request.getParameter("id"));
			if (bundle != null) {	
				try {
					message="Info: "+bundle.getSymbolicName()+" updated";
					bundle.update();
				} catch (BundleException e) {
					message="Error: "+e.getMessage();
				}	
			}
			return new RedirectView(pluginContext.getApplicationPath());		
		}
		else if (pluginContext.getLocalPath().equals("/stop")) {
			Bundle bundle = getBundleById(request.getParameter("id"));
			if (bundle != null) {
				try {
					message="Info: "+bundle.getSymbolicName()+" stoped";
					bundle.stop();
				} catch (BundleException e) {
					message="Error: "+e.getMessage();
				}
			}
			return new RedirectView(pluginContext.getApplicationPath());
		}
		else if (pluginContext.getLocalPath().equals("/start")) {
			Bundle bundle = getBundleById(request.getParameter("id"));
			if (bundle != null) {
				try {
					message="Info: "+bundle.getSymbolicName()+" started";
					bundle.start();
				} catch (BundleException e) {
					message="Error: "+e.getMessage();
				}
			}
			return new RedirectView(pluginContext.getApplicationPath());
		}
		else{
			String temp=message;
			message=null;
			return new BundleListView(context, loader, temp);
		}
	}
	
	private Bundle getBundleById(String idString) {
		if (idString != null) {
			long id = new Long(idString);
			return context.getBundle(id);
		}
		else return null;	
	}
	
	private void installBundle(HttpServletRequest request, PluginContext pluginContext)
	{
		if(ServletFileUpload.isMultipartContent(request)){
			FileItemFactory factory=new DiskFileItemFactory();
			ServletFileUpload upload=new ServletFileUpload(factory);
	
			try {
				List<?> files=upload.parseRequest(request);
				
				byte[] buffer=new byte[8192];
				for(Iterator<?> i=files.iterator();i.hasNext();){
					FileItem element= (FileItem)i.next();
					
					if(!element.isFormField()){
						String fileName=element.getName();
						if(!fileName.endsWith(".jar")){
							throw new FileUploadException("Wrong data type. Needs to be a \".jar\".");
						}
						fileName=fileName.replace('\\','/'); //Windows stub
						if(fileName.contains("/")){
							fileName=fileName.substring('/'+1);
						}
					
						InputStream is=element.getInputStream();
						FileOutputStream fos=new FileOutputStream(new File("bundles",fileName));
						
						int len=0;
						while((len=is.read(buffer)) > 0){
							fos.write(buffer, 0, len);
						}
						
						fos.flush();
						fos.close();
						is.close();
						
						context.installBundle("file:bundles/"+fileName);
						
						message="Info: Installed Bundle "+fileName;
					}
				}
			}
			catch (Exception e) {
				message="Error: "+e.getMessage();
			}
		}
	}
	
	public boolean handleSecurity(HttpServletRequest request,
			HttpServletResponse response) throws IOException {
		return true;
	}

	public URL getResource(String name) {	
		return context.getBundle().getResource(name);
	}

	public String getMimeType(String name) {
		return null;
	}

	public PluginCategory getCategory() {
		return PluginCategory.CONFIGURATION;
	}

}
