package org.openmuc.webconfigurator.internal;

import java.io.IOException;
import java.net.URL;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Hashtable;
import java.util.List;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.openmuc.core.datamanager.config.ConfigurationService;
import org.openmuc.core.datamanager.config.DeviceConfiguration;
import org.openmuc.core.datamanager.config.ChannelConfiguration;
import org.openmuc.core.measurement.Unit;
import org.openmuc.webinterface.MessageView;
import org.openmuc.webinterface.PluginCategory;
import org.openmuc.webinterface.PluginContext;
import org.openmuc.webinterface.RedirectView;
import org.openmuc.webinterface.ResourceLoader;
import org.openmuc.webinterface.View;
import org.openmuc.webinterface.WebInterfacePlugin;
import org.osgi.framework.BundleContext;

public class WebconfiguratorPlugin implements WebInterfacePlugin {

	private static SimpleDateFormat dateFormat = new SimpleDateFormat("yy-MM-dd HH:mm:ss");
	
	private BundleContext context;
	private ResourceLoader loader;
	private ConfigurationService configService;
	
	public WebconfiguratorPlugin(BundleContext context, ConfigurationService configService) {
		this.context = context;
		this.configService = configService;
		loader = new ResourceLoader(context);
	}
	
	public Hashtable<String,String> getResources() {
		Hashtable<String,String> resources = new Hashtable<String,String>();
		
		resources.put("css", "css");
		
		return resources;
	}
	
	public String getApplicationAlias() {
		return "measurementconfig";
	}

	public String getApplicationName() {
		return "Measurement Configuration";
	}
	
	public String getApplicationDescription(){
		return "Configuration utility for the devices and channels.";
	}
	
	public View getContentView(HttpServletRequest request, PluginContext context) {
		if (context.getLocalPath().equals("/deletemeter")) {
			String name = request.getParameter("id");
			configService.deleteDevice(name);
			configService.persist();
			return new RedirectView(context.getApplicationPath() + "/meterlist");
		}
		else if (context.getLocalPath().equals("/editmeter")) {
			String name = request.getParameter("id");
			DeviceConfiguration meterConfig = configService.getDeviceByName(name);
			WebconfigView view = new WebconfigView("Edit meter configuration", 
					loader.getResourceAsString("editmeter.html"));
			
			view.addToContext("meter", meterConfig);
			view.addToContext("meterurl", meterConfig.getUrl().toString());
			return view;
		}
		else if (context.getLocalPath().equals("/editchannel")) {
			String label = request.getParameter("id");
			String name = request.getParameter("meter");
			
			DeviceConfiguration meterConfig = configService.getDeviceByName(name);
			ChannelConfiguration register = meterConfig.getChannelByLabel(label);
						
			WebconfigView view = new WebconfigView("Edit channel configuration", 
					loader.getResourceAsString("editchannel.html"));
			
						
			view.addToContext("channel", register);
			view.addToContext("units", Unit.values());
			view.addToContext("meterName", name);
			view.addToContext("storingStartDate", dateFormat.format(register.getStoringStartTime()));
			view.addToContext("storingType", storageType(register.getStorageType()));
			
			boolean scalerSelected; 
			
			if (register.getScaler() != 1.0) 
				scalerSelected = true; 
			else scalerSelected = false;
			
			view.addToContext("scalerSelected", scalerSelected);
			
			return view;
		}
		else if (context.getLocalPath().equals("/deletechannel")) {
			String label = request.getParameter("id");
			String name = request.getParameter("meter");
			
			DeviceConfiguration meterConfig = configService.getDeviceByName(name);
			meterConfig.deleteChannel(label);
			
			configService.persist(); /* store to persistent configuration database */
			
			return new RedirectView(context.getApplicationPath() + "/showchannels?id=" + name);
		}
		else if (context.getLocalPath().equals("/updatechannel")) {
			return processEditChannelForm(request, context);
		}
		else if (context.getLocalPath().equals("/updatemeter")) {
			return processEditDeviceForm(request, context);
		}
		else if (context.getLocalPath().equals("/addmeter")){
			WebconfigView view = new WebconfigView("Add meter", loader.getResourceAsString("addmeter.html"));
			return view;
		}
		else if (context.getLocalPath().equals("/addchannel")){
			WebconfigView view = new WebconfigView("Add channel", loader.getResourceAsString("addchannel.html"));
			String name = request.getParameter("meter");
			view.addToContext("units", Unit.values());
			view.addToContext("meterName", name);
			return view;
		}
		else if (context.getLocalPath().equals("/showchannels")) {
			String name = request.getParameter("id");
			DeviceConfiguration meterConfig = configService.getDeviceByName(name);
			List<ChannelConfiguration> registers = meterConfig.getChannels();
			
			WebconfigView view = new WebconfigView("Devices", loader.getResourceAsString("channels.html"));
			view.addToContext("meterName", name);
			view.addToContext("channelList", registers);
			view.addToContext("utils", this);
			return view;
		}
		else if (context.getLocalPath().equals("/channellist")){
			List<DeviceConfiguration> devices = configService.getDevices();

			WebconfigView view = new WebconfigView("Channels", loader.getResourceAsString("channellist.html"));
			
			view.addToContext("devices", devices);
			view.addToContext("utils", this);
			
			return view;
		}
		else {
			List<DeviceConfiguration> devices = configService.getDevices();
			
			WebconfigView view = new WebconfigView("Devices", loader.getResourceAsString("devices.html"));
			
			view.addToContext("deviceList", devices);
			return view;
		}
	}

	private View processEditDeviceForm(HttpServletRequest request, PluginContext context) {
		String originalName = request.getParameter("originalname");
		
		String newName = request.getParameter("name");
		DeviceConfiguration meterConfig;
		if(originalName.equals("")){
			if (newName.equals("")) {
				return new MessageView("A device needs a unique name!", MessageView.ERROR);
			}

			if (configService.getDeviceByName(newName) != null)
				return new MessageView("Device " + newName + " already exists!", MessageView.ERROR);

			meterConfig = configService.createDevice(newName);
			originalName=newName;
		}
		else{
			meterConfig = configService.getDeviceByName(originalName);
		}
		
		if (!newName.equals(originalName)) {
			if (newName.equals("")) {
				return new MessageView("A device needs a unique name!", MessageView.ERROR);
			}

			if (configService.getDeviceByName(newName) != null)
				return new MessageView("Device " + newName + " already exists!", MessageView.ERROR);

			meterConfig.setName(newName);
		}
		
		try{
			meterConfig.setSamplingPeriod(new Long(request.getParameter("samplingPeriod")));
		}
		catch(Exception e){
			meterConfig.setSamplingPeriod(1000);
		}
		
		meterConfig.setDescription(request.getParameter("description"));
		meterConfig.setMeterUrl(request.getParameter("meterurl"));
		meterConfig.setVendorID(request.getParameter("vendorid"));
		meterConfig.setDeviceID(request.getParameter("deviceid"));
		configService.persist();
		return new RedirectView(context.getApplicationPath() + "/meterlist");
	}
	
	private View processEditChannelForm(HttpServletRequest request, PluginContext context) {
		String label = request.getParameter("originallabel");
		
		DeviceConfiguration meterConfig = 
			configService.getDeviceByName(request.getParameter("meter"));
		
		ChannelConfiguration register;
		
		String newLabel = request.getParameter("label");
		
		if (newLabel.equals("")) {
			return new MessageView("Label must be a unique name!", MessageView.ERROR);
		}
		
		if(label.equals("")){
			if (isLabelAlreadyExisting(newLabel)) {
				return new MessageView("Label " + newLabel + " already exists!", MessageView.ERROR);
			}
			else {
				register = meterConfig.createChannel(newLabel);
				label=newLabel;
			}
			
		}
		else{
			register = meterConfig.getChannelByLabel(label);
		}
		
		if (!label.equals(newLabel)) {
			if (isLabelAlreadyExisting(newLabel)) {
				return new MessageView("Label " + newLabel + " already exists!", MessageView.ERROR);
			}
			else {
				register.setLabel(newLabel);
			}
		}
		
		if (request.getParameter("scalerChecked") != null) {
			try {
				double scaler = Double.parseDouble(request.getParameter("scaler"));
				register.setScaler(scaler);
			} catch (NumberFormatException e) {
				return new MessageView("Scaler is not a decimal!", MessageView.ERROR);
			}
		}
		else register.setScaler(1.0);
		
		Date storingStartTime;
		
		try {
			System.out.println("starttime:" + request.getParameter("starttime"));
			storingStartTime = dateFormat.parse(request.getParameter("starttime"));
		} catch (ParseException e) {
			return new MessageView("Unable to parse start time: Format is \"YY:MM:HH hh:mm:ss\".", MessageView.ERROR);
		}
		
		register.setDescription(request.getParameter("description"));
		register.setStoringStartTime(storingStartTime);
		register.setStorageType(storageType(request.getParameter("storagetype")));
		register.setUnit(Unit.fromUnitCode(new Integer(request.getParameter("unit"))));
		register.setObjectLocator(request.getParameter("locator"));
		try{
			register.setStoringPeriod(new Integer(request.getParameter("storingperiod")));
		}
		catch(Exception e){
			register.setStoringPeriod(300);
		}

		configService.persist();
		return new RedirectView(context.getApplicationPath() + "/showchannels?id=" 
					+ request.getParameter("meter"));
	}

	private boolean isLabelAlreadyExisting(String newLabel) {
		List<DeviceConfiguration> devices = configService.getDevices();
		
		for (DeviceConfiguration device : devices) {
			for (ChannelConfiguration channel : device.getChannels()) {
				if (channel.getLabel().equals(newLabel)) return true;
			}
		}
		
		return false;
	}
	
	public String storageType (int storageType) {
		switch (storageType) {
		case ChannelConfiguration.STORE_TYPE_INST:
			return "instantaneous";
		case ChannelConfiguration.STORE_TYPE_MEAN:
			return "mean value";
		case ChannelConfiguration.STORE_TYPE_NONE:
			return "none";
		case ChannelConfiguration.STORE_TYPE_VALUE_CHANGED:
			return "value changed";
		default:
			return "unknown/invalid";
		}
	}
	
	public int storageType (String storageType) {
		if (storageType.equals("instantaneous"))
			return ChannelConfiguration.STORE_TYPE_INST;
		if (storageType.equals("mean value"))
			return ChannelConfiguration.STORE_TYPE_MEAN;
		if (storageType.equals("value changed"))
			return ChannelConfiguration.STORE_TYPE_VALUE_CHANGED;
			
		return ChannelConfiguration.STORE_TYPE_NONE;
	}

	public boolean handleSecurity(HttpServletRequest request,
			HttpServletResponse response) throws IOException {
		return true;
	}

	public URL getResource(String name) {	
		return context.getBundle().getResource(name);
	}

	public String getMimeType(String name) {
		return null;
	}

	public PluginCategory getCategory() {
		return PluginCategory.CONFIGURATION;
	}

}
