package org.openmuc.webdataplot.internal;

import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.openmuc.core.datamanager.Authentication;
import org.openmuc.core.datamanager.DataManager;
import org.openmuc.core.datamanager.DataManagerTracker;
import org.openmuc.core.datamanager.DataStorage;
import org.openmuc.core.datamanager.config.ConfigurationService;
import org.openmuc.core.measurement.Value;
import org.openmuc.core.measurement.State;
import org.openmuc.webinterface.AjaxView;
import org.openmuc.webinterface.PluginCategory;
import org.openmuc.webinterface.PluginContext;
import org.openmuc.webinterface.ResourceLoader;
import org.openmuc.webinterface.View;
import org.openmuc.webinterface.WebInterfacePlugin;
import org.osgi.framework.BundleContext;
import flexjson.JSONSerializer;

public class PlotWebToolPlugin implements WebInterfacePlugin {

	private BundleContext context;
	private ResourceLoader loader;
	private DataManagerTracker dmTracker;
	//private DeviceConfigurator deviceConfigurator;
	
	public PlotWebToolPlugin(BundleContext context, ConfigurationService configService) {
		this.context = context;
		this.dmTracker = new DataManagerTracker(context);
    	this.dmTracker.open();
		loader = new ResourceLoader(context);
		//deviceConfigurator = new DeviceConfigurator(configService);
	}
	
	public Hashtable<String,String> getResources() {
		Hashtable<String,String> resources = new Hashtable<String,String>();
		
		resources.put("css", "css");
		resources.put("js", "js");
		resources.put("images", "images");
		
		return resources;
	}
	
	public String getApplicationAlias() {
		return "webdataplot";
	}

	public String getApplicationName() {
		return "Data Plotter";
	}
	
	public String getApplicationDescription() {
		return "A plotter, who prints graphs from the data.";
	}

	public View getContentView(HttpServletRequest request, PluginContext context) {
		String template = "";
		
	    DataManager dm = dmTracker.getDataManager();
	    List<String> labels= dm.getDeviceDirectory();
	    
	    Authentication auth = new Authentication("admin", "admin");		
		
		if (context.getLocalPath().equals("/getData")){							//AJAX request, for data
		    String label = request.getParameter("label");
		    long end=new Long(request.getParameter("end"));
		    long start=new Long(request.getParameter("start"));
		    int resolution=new Integer(request.getParameter("resolution"));
		    DataStorage store;
		    if(labels.contains(label)){
		    	store=dm.getDataStorage(label, auth);
		    }
		    else{
		    	store=dm.getDataStorage(labels.get(0), auth);
		    }
			
		    JSONSerializer serializer = new JSONSerializer();
		    List<Value> values=store.getValues(start,end);
		    
		    
		    int period=store.getConfiguration().getStoringPeriod()*1000;
		    
		    for(int i=1;i<values.size();i++){
		    	if(values.get(i).getTimestamp()-values.get(i-1).getTimestamp()>period){
		    		values.add(i, new Value(0,values.get(i).getTimestamp()-1, State.invalid));
		    		i++;
		    	}
		    }
		    
		    
		    ArrayList<Value> newvalues=new ArrayList<Value>();
		    String serializedJSONContent;
		    
		    if(resolution!=0 && resolution<=values.size()){
			    long diff=(end-start)/(resolution);
				
				int oldi=0;
				for(int i=0;i<values.size();i++){
					if(values.get(i).getTimestamp()>= start + diff|| i+1==values.size()){
						if(i!=oldi){
							newvalues.add(values.get(i));
						}
						
						oldi=i;
	
						start=start+diff;
	
					}
				}
				serializedJSONContent = serializer.serialize(newvalues);
		    }
		    else{
		    	serializedJSONContent = serializer.serialize(values);
		    }
		    AjaxView view = new AjaxView(serializedJSONContent);
		    return view;
		}
		else if (context.getLocalPath().equals("/getLiveData")){				//AJAX request, for livedata
			String label = request.getParameter("label");
		    DataStorage store;
		    if(labels.contains(label)){
		    	store=dm.getDataStorage(label, auth);
		    }
		    else{
		    	store=dm.getDataStorage(labels.get(0), auth);
		    }
		    JSONSerializer serializer = new JSONSerializer();
		    Value value=store.getCurrentValue();
		    String serializedJSONContent = serializer.serialize(value);
		    AjaxView view = new AjaxView(serializedJSONContent);
		    return view;
		}
		else if(context.getLocalPath().equals("/getBarData")){
			String label=request.getParameter("label");
			long start=new Long(request.getParameter("start"));
			long end=new Long(request.getParameter("end"));
			int steps=new Double(request.getParameter("steps")).intValue();
			
			DataStorage store;
			if(labels.contains(label)){
				store=dm.getDataStorage(label, auth);
			}
			else{
				store=dm.getDataStorage(labels.get(0), auth);
			}
			
			List<Value> values=store.getValues(start,end);
			
			ArrayList<Value> barvalues=new ArrayList<Value>();

			long diff=(end-start)/(steps);
			int period=(store.getConfiguration().getStoringPeriod())*1000;
			
			int oldi=0;
			for(int i=0;i<values.size();i++){
				if(values.get(i).getTimestamp()>= start + diff|| i+1==values.size()){
					if(i!=oldi){
						Value temp;
						if(values.get(i-1).getTimestamp()-values.get(oldi).getTimestamp()<diff-period)
						{							//values only valid, if there are data at end and beginning
							temp=new Value(values.get(i-1).getValue()-values.get(oldi).getValue(),start,State.invalid);
						}
						else
						{
							temp=new Value(values.get(i-1).getValue()-values.get(oldi).getValue(),start,State.valid);
						}
						barvalues.add(temp);
					}
					else{
						Value temp = new Value(0,start,State.invalid);
						barvalues.add(temp);
					}
					
					oldi=i;

					start=start+diff;

				}
			}
			while(start<end){				//Fill requested values with nothing
				barvalues.add(new Value(0,start,State.invalid));
				start=start+diff;
			}
			
		    JSONSerializer serializer = new JSONSerializer();
		    String serializedJSONContent = serializer.serialize(barvalues);
		    AjaxView view = new AjaxView(serializedJSONContent);
			return view;
			
			
		}
		else if(context.getLocalPath().equals("/dataPlotter")){					//AJAX request for data Plotter
			template=loader.getResourceAsString("dataPlotter.html");
			AjaxView view = new AjaxView(template);
			view.addToContext("labels", labels);
			return view;
		}
		else if(context.getLocalPath().equals("/livePlotter")){					//AJAX request for live Plotter
			template=loader.getResourceAsString("livePlotter.html");
			AjaxView view = new AjaxView(template);
			view.addToContext("labels", labels);
			return view;
		}
		else if(context.getLocalPath().equals("/barPlotter")){					//AJAX request for bar Plotter
			template=loader.getResourceAsString("barPlotter.html");
			AjaxView view = new AjaxView(template);
			view.addToContext("labels", labels);
			return view;
		}
		else if(context.getLocalPath().equals("/dataLister")){					//AJAX request for data Lister
			template=loader.getResourceAsString("dataLister.html");
			AjaxView view = new AjaxView(template);
			view.addToContext("labels", labels);
			return view;
		}
		else{
			DataStorage store;
			ArrayList<String> description=new ArrayList<String>();
			ArrayList<String> unit=new ArrayList<String>();
			
			for(int i=0;i<labels.size();i++)
			{
				store=dm.getDataStorage(labels.get(i), auth);
				description.add(store.getConfiguration().getDescription());
				unit.add(store.getConfiguration().getUnit().toString());
			}

			template = loader.getResourceAsString("index.html");
			PlotView view = new PlotView(template);
			view.addToContext("labels", labels);
			view.addToContext("description", description);
			view.addToContext("unit", unit);
			return view;
		}
	}
	
	
	public boolean handleSecurity(HttpServletRequest request,
			HttpServletResponse response) throws IOException {
		return true;
	}

	public URL getResource(String name) {	
		return context.getBundle().getResource(name);
	}

	public String getMimeType(String name) {
		return null;
	}

	public PluginCategory getCategory() {
		return PluginCategory.APPLICATION;
	}

}
