package org.openmuc.webexporttool.internal;

import java.io.IOException;
import java.io.PrintWriter;
import java.net.URL;
import java.util.Calendar;
import java.util.Hashtable;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.openmuc.core.datamanager.DataManager;
import org.openmuc.webinterface.MessageView;
import org.openmuc.webinterface.PluginCategory;
import org.openmuc.webinterface.PluginContext;
import org.openmuc.webinterface.ResourceLoader;
import org.openmuc.webinterface.View;
import org.openmuc.webinterface.WebInterfacePlugin;
import org.osgi.framework.BundleContext;

import static java.util.Calendar.*;

public class WebExportToolPlugin implements WebInterfacePlugin {

	private BundleContext context;
	private ResourceLoader loader;
	private DataManagerTracker dmTracker;
	
	public WebExportToolPlugin(BundleContext context, DataManagerTracker dmTracker) {
		this.context = context;
		this.dmTracker = dmTracker;
		loader = new ResourceLoader(context);
	}
	
	public Hashtable<String,String> getResources() {
		Hashtable<String,String> resources = new Hashtable<String,String>();
		
		resources.put("css", "css");
		
		return resources;
	}
	
	public String getApplicationAlias() {
		return "dataexporttool";
	}

	public String getApplicationName() {
		return "Data Export Tool";
	}
	
	public String getApplicationDescription(){
		return "A tool to export data as CSV table.";
	}

	public View getContentView(HttpServletRequest request, PluginContext context) {
		String template = "";
		
		if (context.getLocalPath().equals("/exportdata")) {
			return handleExportForm(context);
		}
		else {
			DataManager manager = dmTracker.getDataManager();
			
			if (manager == null) {
				return new MessageView("Data currently not available!", MessageView.ERROR);
			}
			
			template = loader.getResourceAsString("exportform.html");
			WebExportView view = new WebExportView(template);
			
			Calendar now = Calendar.getInstance();
			
			view.addToContext("startDaySelected", new Integer(now.get(DAY_OF_MONTH)));
			view.addToContext("startMonthSelected", new Integer(now.get(MONTH) + 1));
			view.addToContext("startYearSelected", new Integer(now.get(YEAR)));
			view.addToContext("labellist", manager.getDeviceDirectory());
			
			return view; 
		}
	}
	
	private View handleExportForm(PluginContext context) {
		HttpServletRequest request = context.getRequest();
		
		String[] labels = request.getParameterValues("labellist");
		
		if (labels == null)
			return new MessageView("Please select at least on channel!", MessageView.ERROR);
			
		try {
			int startDay = new Integer(request.getParameter("startdateday"));
			int startMonth = new Integer(request.getParameter("startdatemonth"));
			int startYear = new Integer(request.getParameter("startdateyear"));
			
			int dateFormat = new Integer(request.getParameter("timeformat"));
			
			long end;
			
			if (request.getParameter("enddate") != null) {
				int endDay = new Integer(request.getParameter("enddateday"));
				int endMonth = new Integer(request.getParameter("enddatemonth"));
				int endYear = new Integer(request.getParameter("enddateyear"));
				
				Calendar cal = Calendar.getInstance();
				cal.clear();
				
				cal.set(endYear, endMonth - 1, endDay);
				cal.add(DAY_OF_MONTH, 1);
				
				end = cal.getTimeInMillis();
			}
			else
				end = System.currentTimeMillis();
			
			Calendar cal = Calendar.getInstance();
			
			cal.clear();
			cal.set(startYear, startMonth - 1, startDay);
			long start = cal.getTimeInMillis();
			
			DataManager manager = dmTracker.getDataManager();
			
			CSVDataExport exporter = new CSVDataExport(manager);
			
			HttpServletResponse response = context.getResponse();
			
			PrintWriter pw;
			try {
				pw = response.getWriter();
				response.setContentType("application/octet-stream");
				response.setHeader("Content-Disposition", "attachment;filename=exporteddata.csv");
						
				exporter.exportData(labels, pw, start, end, dateFormat);
				pw.flush();
				
				return new MessageView("Data exported.", MessageView.INFO);
			} catch (IOException e) {
				e.printStackTrace();
			}

		} catch (NumberFormatException e) {
			e.printStackTrace();
			return new MessageView("Invalid date format!", MessageView.ERROR);
		}
		
		return new MessageView("Function not available in evaluation version!", MessageView.WARNING);
	}

	public boolean handleSecurity(HttpServletRequest request,
			HttpServletResponse response) throws IOException {
		return true;
	}

	public URL getResource(String name) {	
		return context.getBundle().getResource(name);
	}

	public String getMimeType(String name) {
		return null;
	}

	public PluginCategory getCategory() {
		return PluginCategory.TOOL;
	}

}
