package org.openmuc.webinterface.internal;

import org.apache.velocity.app.Velocity;
import org.openmuc.webinterface.ResourceLoader;
import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;
import org.osgi.service.http.HttpService;

public final class Activator implements BundleActivator {
	private PluginManager pluginManager;

	public void start(BundleContext context) throws Exception {

		ResourceLoader loader = new ResourceLoader(context);

		Velocity.setProperty("resource.loader", "class");
		Velocity.setProperty("class.resource.loader.class",
				"org.apache.velocity.runtime.resource.loader.ClasspathResourceLoader");

		try {
			Velocity.init();
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}

		ServiceReference httpServiceRef = context.getServiceReference(HttpService.class.getName());

		if (httpServiceRef != null) {
			HttpService httpService = (HttpService) context.getService(httpServiceRef);

			pluginManager = new PluginManager(context, httpService);

			httpService.registerServlet("/openmuc", new WebFrontController(loader, pluginManager), null, null);
			httpService.registerResources("/openmuc/css", "/css", null);
			httpService.registerResources("/openmuc/images", "/images", null);
			httpService.registerResources("/openmuc/js", "/js", null);

			pluginManager.open();
		}
		else
			throw new Exception("HttpService not present!");

	}

	public void stop(BundleContext context) throws Exception {
		ServiceReference httpServiceRef = context.getServiceReference(HttpService.class.getName());

		if (httpServiceRef != null) {
			HttpService httpService = (HttpService) context.getService(httpServiceRef);
			httpService.unregister("/openmuc");
		}

		pluginManager.close();
	}
}
