package org.openmuc.webinterface.internal;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.HashMap;

public class Authentication {

	public String path;
	
	public Authentication(){
		path = System.getProperty("bundles.configuration.location");
		if (path == null) {
			path = "shadow";
		}
		else {
			if (path.endsWith("/")){
				path = path.substring(0, path.length()-1);
			}
			path += "/shadow";
		}
		File file = new File(path);
		if(!file.exists()){
			register("admin", "admin");
		}
	}
	
	public void register(String user, String pwd){
		pwd += generateHash(user);							//use the hash of the username as salt
		
		String hash = generateHash(pwd);
		
		setUser(user, hash);
	}
	
	public boolean login(String user, String pwd){
		boolean isAuthenticated = false;
		HashMap<String, String> shadow = getShadow();

		pwd += generateHash(user);							//use the hash of the username as salt
		String hash = generateHash(pwd);
		if(shadow.containsKey(user)){
			String storedHash = shadow.get(user);
			if(hash.equals(storedHash)){
				isAuthenticated = true;
			}
		}
		
		return isAuthenticated;
	}

	public void delete(String user) {
		HashMap<String, String> shadow = getShadow();
		
		shadow.remove(user);

		writeShadow(shadow);
	}

	public boolean contains(String user) {
		HashMap<String, String> shadow = getShadow();
		return shadow.containsKey(user);
	}
	
	private String generateHash(String pwd){
		StringBuilder hash = new StringBuilder();
		
		MessageDigest sha;
		try {
			sha = MessageDigest.getInstance("SHA-256");
			byte[] hashedBytes = sha.digest(pwd.getBytes());
			char[] digits = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f'};
			
			for(byte hashedByte : hashedBytes){
				hash.append(digits[(hashedByte & 0xf0) >> 4]);
				hash.append(digits[(hashedByte & 0x0f)]);
			}
		
		}
		catch (NoSuchAlgorithmException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return hash.toString();
	}
	
	private void setUser(String user, String hash){
		HashMap<String, String> shadow = getShadow();
		shadow.put(user, hash);

		writeShadow(shadow);
	}
	
	private void writeShadow(HashMap<String, String> shadow){
		String text="";
		
		for(String key : shadow.keySet()){
			text += key + ":" + shadow.get(key) + "\n";
		}
		try {
			Writer output = new BufferedWriter(new FileWriter(new File(path)));
			output.write(text);
			output.flush();
			output.close();
		}
		catch (IOException e) { }
		
	}
	
	private HashMap<String, String> getShadow(){
		HashMap<String, String> shadow = new HashMap<String, String>();
		try {
			BufferedReader reader = new BufferedReader(new FileReader(path));
			
			String line = "";
			
			while((line = reader.readLine()) != null){
				String[] temp = line.split(":");
				
				shadow.put(temp[0], temp[1]);
			}
			
		} 
		catch (Exception e) { }
		
		return shadow;
	}
}
