package org.openmuc.webinterface.internal;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.openmuc.webinterface.PluginCategory;
import org.openmuc.webinterface.PluginContext;

public class PluginContextImpl implements PluginContext {

	private String applicationPath;
	private String applicationAlias;
	private String localPath;
	private HttpServletRequest request;
	private HttpServletResponse response;
	
	public PluginContextImpl(String pathInfo, HttpServletRequest request, 
			HttpServletResponse response) 
	{
		extractPathInfoParts(pathInfo);
		this.request = request;
		this.response = response;
	}
	
	public String getApplicationPath() {
		return applicationPath;
	}

	public String getLocalPath() {
		return localPath;
	}

	public String getApplicationAlias() {
		return applicationAlias;
	}
	
	private void extractPathInfoParts(String pathInfo) {
		String categoryPrefixes[] = {"/" + PluginCategory.APPLICATION.toString() + "/",
								 	 "/" + PluginCategory.CONFIGURATION.toString() + "/",
								 	 "/" + PluginCategory.TOOL.toString() + "/"};
		
		for (String categoryPrefix : categoryPrefixes) {
			if (pathInfo.startsWith(categoryPrefix)) {
				String applicationPath = pathInfo.replaceFirst(categoryPrefix, "");
	
				int slashIndex = applicationPath.indexOf('/');
				
				if (slashIndex != -1) {
					applicationAlias = applicationPath.substring(0, slashIndex);
					localPath = applicationPath.substring(slashIndex);
				}
				else {
					applicationAlias = applicationPath;
					localPath = "";
				}
				
				this.applicationPath = "/openmuc" + categoryPrefix + applicationAlias;
				return;
			}
		}
		
		applicationAlias = "";
		localPath = "";
		applicationPath = "";
	}

	public HttpServletResponse getResponse() {
		return response;
	}

	public HttpServletRequest getRequest() {
		return request;
	}
}
