package org.openmuc.webinterface.internal;

import java.util.Hashtable;
import java.util.Set;

import org.apache.log4j.Logger;
import org.openmuc.webinterface.WebInterfacePlugin;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;
import org.osgi.service.http.HttpService;
import org.osgi.service.http.NamespaceException;
import org.osgi.util.tracker.ServiceTracker;

public class PluginManager extends ServiceTracker {
	
	private static Logger logger = Logger.getLogger(PluginManager.class);
	
	private Hashtable<String,WebInterfacePlugin> plugins = new Hashtable<String,WebInterfacePlugin>(); 
	
	private HttpService httpService;
	
	public PluginManager(BundleContext context, HttpService httpService) {	
		super(context, WebInterfacePlugin.class.getName(), null);
		this.httpService = httpService;
	}
	
	private void registerResources(WebInterfacePlugin plugin)
	{
		Set<String> aliases = plugin.getResources().keySet();
		
		for (String alias : aliases) {
			try {

				httpService.registerResources(
						"/openmuc/" + plugin.getCategory().toString() + "/" + plugin.getApplicationAlias()
								+ "/" + alias, plugin.getResources().get(alias), plugin);

			} catch (NamespaceException e) {
				e.printStackTrace();
			}
		}
	}

	private void unregisterResources(WebInterfacePlugin plugin)
	{
		Set<String> aliases = plugin.getResources().keySet();
		
		for (String alias : aliases) {
			httpService.unregister("/openmuc/" + plugin.getCategory().toString() + "/"
					+ plugin.getApplicationAlias() + "/" + alias);
		}
	}
	
	@Override
	public Object addingService(ServiceReference reference) {
		WebInterfacePlugin plugin = (WebInterfacePlugin) context.getService(reference);
		
		logger.info("Found new plugin: " + plugin.getApplicationName());
		
		plugins.put(plugin.getApplicationAlias(), plugin);
		registerResources(plugin);
		
		return plugin;
	}

	@Override
	public void removedService(ServiceReference reference, Object service) {
		WebInterfacePlugin plugin = (WebInterfacePlugin) service;
		
		logger.info("Remove plugin: " + plugin.getApplicationName());
		
		unregisterResources(plugin);
		plugins.remove(plugin.getApplicationAlias());
		
		context.ungetService(reference);
	}
	
	public WebInterfacePlugin getPlugin(String alias) {
		return plugins.get(alias);
	}

	public Hashtable<String,WebInterfacePlugin> getPlugins() {
		return plugins;
	}
}
