package org.openmuc.webmeasurement.internal;

import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.openmuc.core.datamanager.Authentication;
import org.openmuc.core.datamanager.DataManager;
import org.openmuc.core.datamanager.DataManagerTracker;
import org.openmuc.core.datamanager.DataStorage;
import org.openmuc.core.datamanager.config.ConfigurationService;
import org.openmuc.core.datamanager.config.DeviceConfiguration;
import org.openmuc.core.measurement.Value;
import org.openmuc.webinterface.AjaxView;
import org.openmuc.webinterface.PluginCategory;
import org.openmuc.webinterface.PluginContext;
import org.openmuc.webinterface.RedirectView;
import org.openmuc.webinterface.ResourceLoader;
import org.openmuc.webinterface.View;
import org.openmuc.webinterface.WebInterfacePlugin;
import org.osgi.framework.BundleContext;
import flexjson.JSONSerializer;

public class MeasureToolPlugin implements WebInterfacePlugin {

	private BundleContext context;
	private ResourceLoader loader;
	private DataManagerTracker dmTracker;
	private ConfigurationService configService;
	
	public MeasureToolPlugin(BundleContext context, ConfigurationService configService) {
		this.context = context;
		this.dmTracker = new DataManagerTracker(context);
    	this.dmTracker.open();
    	this.configService=configService;
		loader = new ResourceLoader(context);
	}
	
	public Hashtable<String,String> getResources() {
		Hashtable<String,String> resources = new Hashtable<String,String>();
		
		resources.put("css", "css");
		resources.put("js", "js");
		resources.put("images", "images");
		
		return resources;
	}
	
	public String getApplicationAlias() {
		return "webmeasurement";
	}

	public String getApplicationName() {
		return "Measurement";
	}
	
	public String getApplicationDescription() {
		return "Shows the current values of the sensors, and allows you to set output values.";
	}

	public View getContentView(HttpServletRequest request, PluginContext context) {
		String template = "";	
		
		if (context.getLocalPath().equals("/getValue")) {
			String label = request.getParameter("label");
		    Value value=getStore(label).getCurrentValue();
		    JSONSerializer serializer = new JSONSerializer();
		    String serializedJSONContent = serializer.serialize(value);
		    AjaxView view = new AjaxView(serializedJSONContent);
		    return view;
		}
		if(context.getLocalPath().equals("/setValue")) {
			String label = request.getParameter("label");
			Double value;
			try{
				value = new Double(request.getParameter("value"));
			}
			catch(NumberFormatException e){
				value = 0.0;
			}
		    getStore(label).setValue(value);
		    return new RedirectView(context.getApplicationPath());
		}
		if(context.getLocalPath().equals("/measure")) {
			try {
				String[] sDevices = request.getParameterValues("devices");
				List<DeviceConfiguration> devices = new ArrayList<DeviceConfiguration>();
				for(String device : sDevices) {
					devices.add(configService.getDeviceByName(device));
				}
				template = loader.getResourceAsString("measure.html");
				MeasureView view = new MeasureView(template);
				view.addToContext("devices", devices);
				return view;
			}
			catch(Exception e) {
				return new RedirectView(context.getApplicationPath());
			}
		}
		else {
			List<DeviceConfiguration> devices = configService.getDevices();
			
			template = loader.getResourceAsString("index.html");
			MeasureView view = new MeasureView(template);
			view.addToContext("devices", devices);
			return view;
		}
	}
	
	private DataStorage getStore(String label){
		DataManager dm = dmTracker.getDataManager();
	    List<String> labels= dm.getDeviceDirectory();
	    Authentication auth = new Authentication("admin", "admin");
	    DataStorage store;
		if(labels.contains(label)){
	    	store=dm.getDataStorage(label, auth);
	    }
	    else{
	    	store=dm.getDataStorage(labels.get(0), auth);
	    }
		return store;
	}
	
	public boolean handleSecurity(HttpServletRequest request,
			HttpServletResponse response) throws IOException {
		return true;
	}

	public URL getResource(String name) {	
		return context.getBundle().getResource(name);
	}

	public String getMimeType(String name) {
		return null;
	}

	public PluginCategory getCategory() {
		return PluginCategory.APPLICATION;
	}

}
